@@ -44,6 +44,7 @@ __fastcall TForm1::~TForm1()
 {
 	list <SWDisplay *>::iterator it;
 	int loop;
+
 	
 	if (mainmgr)
 		delete mainmgr;
@@ -240,13 +241,44 @@ void __fastcall TForm1::DictKeyEditChange(TObject *Sender)
 		DefaultStrKey = DictKeyEdit->Text.c_str();
 		RefreshActiveSheet(PageControl3);
 		DictKeyEdit->Modified = false;
+		FillDictKeys();
 	}
 }
+
+void TForm1::FillDictKeys() {
+	ModMap::iterator it;
+	int index = 0;
+	SWKey saveKey;
+	
+	it = mainmgr->Modules.find(PageControl3->ActivePage->Caption.c_str());
+	if (it != mainmgr->Modules.end()) {
+		SWModule *mod = (*it).second;
+		mod->KeyText(); // snap to entry
+		saveKey = mod->KeyText();
+		int count = lbDictKeys->Height / lbDictKeys->ItemHeight;
+		lbDictKeys->Items->Clear();
+		for (int i = 0; i < (count / 2); i++)
+			(*mod)++;
+		for (int i = 0; i < count-1; i++)
+			(*mod)--;
+		mod->Error();
+		for (;!mod->Error() && count;count--,(*mod)++) {
+			lbDictKeys->Items->Add(mod->KeyText());
+			if (saveKey == mod->Key())
+				index = lbDictKeys->Items->Count-1;
+		}
+		lbDictKeys->ItemIndex = index;
+	}
+}
+
+
 //---------------------------------------------------------------------------
 void __fastcall TForm1::PageControl3Change(TObject *Sender)
 {
+	DefaultStrKey = DictKeyEdit->Text.c_str();
 	RefreshActiveSheet(PageControl3);
 	Form1->ActiveControl = PageControl3;
+	FillDictKeys();
 }
 //---------------------------------------------------------------------------
 void __fastcall TForm1::NewSearchWindow1Click(TObject *Sender)
@@ -266,16 +298,20 @@ void __fastcall TForm1::About1Click(TObject *Sender)
 //---------------------------------------------------------------------------
 void __fastcall TForm1::DictionaryLookup1Click(TObject *Sender)
 {
-	int wordstart;
-	
 	SWDispRTF *rtf = (SWDispRTF *)Screen->ActiveControl;
+	int wordstart;
+	int saveSelStart = rtf->SelStart;
+	
 	if (!rtf->SelLength) {
 		for (wordstart = rtf->SelStart; (wordstart > -1)&&(isdigit(rtf->Text[wordstart])||isalpha(rtf->Text[wordstart])); wordstart--);
 		rtf->SelStart = (wordstart > -1) ? wordstart: 0;
 		for (wordstart = rtf->SelStart + 1; (wordstart < rtf->Text.Length())&&(isdigit(rtf->Text[wordstart])||isalpha(rtf->Text[wordstart])); wordstart++);
-		rtf->SelLength = ((wordstart < rtf->Text.Length()) ? wordstart : rtf->Text.Length() - 1) - rtf->SelStart - 1;
+		rtf->SelLength = ((wordstart < rtf->Text.Length()) ? wordstart : rtf->Text.Length()) - rtf->SelStart - 1;
+		DictKeyEdit->Text = Trim(rtf->SelText);
+		rtf->SelStart = saveSelStart;
+		rtf->SelLength = 0;
 	}
-	DictKeyEdit->Text = Trim(rtf->SelText);
+	else	DictKeyEdit->Text = Trim(rtf->SelText);
 }
 //---------------------------------------------------------------------------
 void __fastcall TForm1::SaveLayout1Click(TObject *Sender)
@@ -297,10 +333,9 @@ void __fastcall TForm1::FormShow(TObject *Sender)
 {
 	ModMap::iterator it;
 	SectionMap::iterator sit;
-	ConfigEntMap::iterator eit;
+	ConfigEntMap::iterator eit, eitend;
 	AnsiString s1;
 	char *font;
-	int val;
 	int loop;
 	int maxFiles;
 	string tmpval;
@@ -364,12 +399,30 @@ void __fastcall TForm1::FormShow(TObject *Sender)
 			pnlText->Width = atoi((*eit).second.c_str());
 	}
 
+	if ((sit = layoutconf->Sections.find("History")) != layoutconf->Sections.end()) {
+
+		eitend = (*sit).second.upper_bound("SearchText");
+		for (eit = (*sit).second.lower_bound("SearchText"); eit != eitend; eit++)
+			Form2->SearchText->Items->Add(eit->second.c_str());
+
+		eitend = (*sit).second.upper_bound("LookupText");
+		for (eit = (*sit).second.lower_bound("LookupText"); eit != eitend; eit++)
+			freeHandLookup->Items->Add(eit->second.c_str());
+	}
+
 	for (loop = 0; loop < 10; loop++)
 		ctrlstates.insert(ctrlstates.begin(), 0);
-
-	ImageList1->Add(BackBtnImage->Picture->Bitmap, NULL);
-	ImageList1->Add(SearchBtnImage->Picture->Bitmap, NULL);
-	ImageList2->Add(BookmarkBtnImage->Picture->Bitmap, NULL);
+	Graphics::TBitmap *bitmap = BackBtnImage->Picture->Bitmap;
+	TColor transColor = bitmap->Canvas->Pixels[0][0];
+	ImageList1->AddMasked(bitmap, transColor);
+	
+	bitmap = SearchBtnImage->Picture->Bitmap;
+	transColor = bitmap->Canvas->Pixels[0][0];
+	ImageList1->AddMasked(bitmap, transColor);
+	
+	bitmap = BookmarkBtnImage->Picture->Bitmap;
+	transColor = bitmap->Canvas->Pixels[0][0];
+	ImageList2->AddMasked(bitmap, transColor);
 
 	// Add options to Options Main Menu choice
 	OptionsList options = mainmgr->getGlobalOptions();
@@ -523,7 +576,7 @@ void __fastcall TForm1::Copy1Click(TObject *Sender)
 		for (wordstart = rtf->SelStart; (wordstart > -1)&&(isdigit(rtf->Text[wordstart])||isalpha(rtf->Text[wordstart])); wordstart--);
 		rtf->SelStart = (wordstart > -1) ? wordstart: 0;
 		for (wordstart = rtf->SelStart + 1; (wordstart < rtf->Text.Length())&&(isdigit(rtf->Text[wordstart])||isalpha(rtf->Text[wordstart])); wordstart++);
-		rtf->SelLength = ((wordstart < rtf->Text.Length()) ? wordstart : rtf->Text.Length() - 1) - rtf->SelStart - 1;
+		rtf->SelLength = ((wordstart < rtf->Text.Length()) ? wordstart : rtf->Text.Length()) - rtf->SelStart - 1;
 	}
 	rtf->CopyToClipboard();
 }
@@ -767,16 +820,26 @@ void __fastcall TForm1::createVerseList(TObject *Sender)
 }
 //---------------------------------------------------------------------------
 
-void __fastcall TForm1::freeHandLookupKeyPress(TObject *Sender, char &Key)
+void __fastcall TForm1::freeHandLookupoldKeyPress(TObject *Sender, char &Key)
 {
 	if (Key == '\r') {
 		ListKey &tmpVerseList = VerseKey::ParseVerseList(freeHandLookup->Text.c_str(), DefaultVSKey);
-		if (tmpVerseList.Count() > 1) {
-			TVerseListFrm *tmpForm = new TVerseListFrm(this, tmpVerseList);
-			tmpForm->Caption = "User Verse List";
-			tmpForm->Show();
-		}
+
 		if (tmpVerseList.Count()) {
+			int index = freeHandLookup->Items->IndexOf(freeHandLookup->Text);
+			freeHandLookup->Items->Insert(0, freeHandLookup->Text);
+
+			if (index >= 0)
+				freeHandLookup->Items->Delete(index+1);
+		
+			freeHandLookup->ItemIndex = 0;
+
+			if (tmpVerseList.Count() > 1) {
+				TVerseListFrm *tmpForm = new TVerseListFrm(this, tmpVerseList);
+				tmpForm->Caption = "User Verse List";
+				tmpForm->Show();
+			}
+
 			tmpVerseList = TOP;
 			DefaultVSKey = tmpVerseList;
 			TextKeyChanged();
@@ -785,17 +848,23 @@ void __fastcall TForm1::freeHandLookupKeyPress(TObject *Sender, char &Key)
 }
 //---------------------------------------------------------------------------
 
-void __fastcall TForm1::RTFMouseDown(TObject *Sender, TMouseButton Button,
-	 TShiftState Shift, int X, int Y)
-{
+
+void __fastcall TForm1::RTFMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift, int X, int Y) {
 	((TWinControl *)Sender)->SetFocus();
+	TRichEdit *rtf = (TRichEdit *)Sender;
+	if (Button == mbRight) {
+		if (!rtf->SelLength)
+			SendMessage(rtf->Handle, WM_LBUTTONDOWN, MK_LBUTTON, MAKELONG(X, Y));
+	}
 }
+
+
 //---------------------------------------------------------------------------
 
 void __fastcall TForm1::EditEntry1Click(TObject *Sender)
 {
 	SWDispRTF *rtf = (SWDispRTF *)Screen->ActiveControl;
-	SWModule *module = 0;
+	SWModule *module;
 //	EditEntry->RichEdit1->Text = rtf->Text;
 	ModMap::iterator target;
 	target = mainmgr->Modules.find(((TPageControl*)rtf->Parent->Parent)->ActivePage->Caption.c_str());
@@ -817,7 +886,7 @@ void __fastcall TForm1::EditEntry1Click(TObject *Sender)
 void __fastcall TForm1::DeleteEntry1Click(TObject *Sender)
 {
 	SWDispRTF *rtf = (SWDispRTF *)Screen->ActiveControl;
-	SWModule *module = 0;
+	SWModule *module;
 	ModMap::iterator target;
 
 	target = mainmgr->Modules.find(((TPageControl*)rtf->Parent->Parent)->ActivePage->Caption.c_str());
@@ -833,7 +902,7 @@ void __fastcall TForm1::LinktoVerse1Click(TObject *Sender)
 {
 	if (VerseSelFrm->ShowModal() == mrOk) {
 		SWDispRTF *rtf = (SWDispRTF *)Screen->ActiveControl;
-		SWModule *module = 0;
+		SWModule *module;
 		ModMap::iterator target;
 		SWKey *linkkey = new VerseKey(VerseSelFrm->Panel2->Caption.c_str());
 
@@ -887,3 +956,43 @@ void __fastcall TForm1::Contents1Click(TObject *Sender)
 }
 //---------------------------------------------------------------------------
 
+
+void __fastcall TForm1::FormClose(TObject *Sender, TCloseAction &Action)
+{
+	int saveCount = 5;
+	
+
+	ConfigEntMap section;
+	ConfigEntMap::iterator entryIt;
+	
+	section = layoutconf->Sections["History"];
+	entryIt = section.find("SearchSaveCount");
+	if (entryIt != section.end())
+		saveCount = atoi(entryIt->second.c_str());
+	else	section.insert(ConfigEntMap::value_type("SearchSaveCount", "5"));
+
+	section.erase("SearchText");
+	for (int loop = 0; ((loop < saveCount) && (loop <	Form2->SearchText->Items->Count)); loop++)
+		 section.insert(ConfigEntMap::value_type("SearchText", Form2->SearchText->Items->Strings[loop].c_str()));
+
+	entryIt = section.find("LookupSaveCount");
+	if (entryIt != section.end())
+		saveCount = atoi(entryIt->second.c_str());
+	else	section.insert(ConfigEntMap::value_type("LookupSaveCount", "5"));
+
+	section.erase("LookupText");
+	for (int loop = 0; ((loop < saveCount) && (loop <	freeHandLookup->Items->Count)); loop++)
+		 section.insert(ConfigEntMap::value_type("LookupText", freeHandLookup->Items->Strings[loop].c_str()));
+
+	layoutconf->Sections["History"] = section;
+	layoutconf->Save();
+	
+}
+//---------------------------------------------------------------------------
+
+void __fastcall TForm1::lbDictKeysClick(TObject *Sender)
+{
+	DictKeyEdit->Text = lbDictKeys->Items->Strings[lbDictKeys->ItemIndex];
+}
+//---------------------------------------------------------------------------
+
