@@ -7,6 +7,7 @@ USELIB("..\BibleCS\swordlib.lib");
 USEFORM("StatusFrm.cpp", StatusForm);
 USEFORM("RemoteMntFrm.cpp", RemoteMntForm);
 USEFORM("InfoFrm.cpp", InfoForm);
+USEFORM("cipherfrm.cpp", CipherForm);
 //---------------------------------------------------------------------------
 WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int)
 {
@@ -18,6 +19,7 @@ WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int)
 		Application->CreateForm(__classid(TStatusForm), &StatusForm);
 		Application->CreateForm(__classid(TRemoteMntForm), &RemoteMntForm);
 		Application->CreateForm(__classid(TInfoForm), &InfoForm);
+		Application->CreateForm(__classid(TCipherForm), &CipherForm);
 		Application->Run();
 	}
 	catch (Exception &exception)
