@@ -6,6 +6,7 @@
 #include "StatusFrm.h"
 #include "RemoteMntFrm.h"
 #include "InfoFrm.h"
+#include "cipherfrm.h"
 //---------------------------------------------------------------------------
 #pragma package(smart_init)
 #pragma resource "*.dfm"
@@ -258,15 +259,10 @@ void TMainForm::fillSourceTree(const char *sourceConf, TTreeView *tree)
 	string nodeName;
 	string targetVersion;
 	string sourceVersion;
+	bool cipher;
 	
 	SWMgr *mgr;
-/*
-	if (tree == localTree)
-		mgr = &localMgr;
-	else mgr = &((InstallSourceTab *) MainForm->PageControl1->ActivePage->Controls[0])->mgr;
-	if (*mgr)
-		delete (*mgr);
-*/
+
 	if (tree == localTree) {
 		if (localMgr)
 			delete localMgr;
@@ -286,6 +282,8 @@ void TMainForm::fillSourceTree(const char *sourceConf, TTreeView *tree)
 
 	for (sections = mgr->config->Sections.begin(); sections != mgr->config->Sections.end(); sections++) {
 	
+		cipher = false;
+		
 		if (!strcmp(sections->first.c_str(), "Globals"))	// skip [Globals]
 			continue;
 			
@@ -304,6 +302,10 @@ void TMainForm::fillSourceTree(const char *sourceConf, TTreeView *tree)
 		targetVersion = "0.0";
 		sourceVersion = "1.0";
 		
+		entry = sections->second.find("CipherKey");
+		if (entry != sections->second.end())
+			cipher = true;
+		
 		entry = sections->second.find("Version");
 		if (entry != sections->second.end())
 			sourceVersion = entry->second.c_str();
@@ -336,13 +338,16 @@ void TMainForm::fillSourceTree(const char *sourceConf, TTreeView *tree)
 			if (VersionInfo(targetVersion.c_str()) < VersionInfo("1.0")) {
 				node->ImageIndex = 1;
 				node->SelectedIndex = 1;
-				node->StateIndex = 0;
 			}
 			else {
 				node->ImageIndex = 2;
 				node->SelectedIndex = 2;
-				node->StateIndex = 0;
 			}
+			if (cipher) {
+				node->ImageIndex += 2;
+				node->SelectedIndex += 2;
+			}
+			node->StateIndex = 0;
 		}
 	}
 }
@@ -428,11 +433,15 @@ int TMainForm::removeModule(const char *modName)
 {
 	SectionMap::iterator module;
 	ConfigEntMap::iterator fileBegin;
-	ConfigEntMap::iterator fileEnd;
+	ConfigEntMap::iterator fileEnd, entry;
 
 	module = manager->config->Sections.find(modName);
 
 	if (module != manager->config->Sections.end()) {
+		entry = module->second.find("CipherKey");
+		if (entry != module->second.end())
+			CipherForm->cipherEdit->Text = entry->second.c_str();
+			
 		fileBegin = module->second.lower_bound("File");
 		fileEnd = module->second.upper_bound("File");
 
@@ -580,12 +589,14 @@ int TMainForm::copyFile(const char *sourceFile, const char *targetFile)
 
 int TMainForm::installModule(const char *modName, InstallSourceTab *ist)
 {
-	SectionMap::iterator module;
+	SectionMap::iterator module, section;
 	ConfigEntMap::iterator fileBegin;
 	ConfigEntMap::iterator fileEnd;
+	ConfigEntMap::iterator entry;
 	string sourceDir;
 	string buffer;
 	bool aborted = false;
+	bool cipher = false;
 
 	if (ist)
 		sourceDir = "./sources/" + ist->Source;
@@ -596,6 +607,11 @@ int TMainForm::installModule(const char *modName, InstallSourceTab *ist)
 	module = mgr.config->Sections.find(modName);
 
 	if (module != mgr.config->Sections.end()) {
+	
+		entry = module->second.find("CipherKey");
+		if (entry != module->second.end())
+			cipher = true;
+		
 		fileEnd = module->second.upper_bound("File");
 		fileBegin = module->second.lower_bound("File");
 
@@ -710,6 +726,11 @@ int TMainForm::installModule(const char *modName, InstallSourceTab *ist)
 									string targetFile = "./mods.d/";
 									targetFile += ent->d_name;
 									copyFile(modFile.c_str(), targetFile.c_str());
+									if (cipher) {
+										CipherForm->modName = modName;
+										CipherForm->confFile = targetFile;
+										CipherForm->ShowModal();
+									}
 								}
 								else	delete config;
 							}
@@ -736,7 +757,7 @@ void	TMainForm::fillAllSourceTrees()
 }
 
 
-void __fastcall TMainForm::Button4Click(TObject *Sender)
+void __fastcall TMainForm::Button4Click(TObject *Sender)	// REMOVE MODULE
 {
 	TTreeNode *node = installTree->Selected;
 	if (node) {
@@ -777,13 +798,16 @@ void __fastcall TMainForm::Button2Click(TObject *Sender)	// INSTALL
 		if (node->StateIndex == 1)
 			count++;
 	}	// do true progress bar
-	
+
+	if (!count) return;		// if nothing is selected, do nothing
+		
 	progressBar->Max = count;
 	progressBar->Position = 0;
 
 	for (node = tree->Items->GetFirstNode(); node; node = node->GetNext()) {
 		if (node->StateIndex == 1) {	// if selected for install
-			if (node->ImageIndex == 2) {	// if this is an upgrade
+			CipherForm->cipherEdit->Text = "";
+			if ((node->ImageIndex == 2) || (node->ImageIndex == 4)) {	// if this is an upgrade
 				removeModule((const char *)node->Data);
 			}
 				// install module
